//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Base/Axis/Bin.h
//! @brief     Defines structs Bin1D, Bin1DCVector
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_BASE_AXIS_BIN_H
#define BORNAGAIN_BASE_AXIS_BIN_H

#include "Base/Vector/Vectors3D.h"

struct Bin1D {
    Bin1D() = default;
    Bin1D(double lower, double upper) : m_lower(lower), m_upper(upper) {}
    double m_lower{0}; //!< lower bound of the bin
    double m_upper{0}; //!< upper bound of the bin
    double center() const { return (m_lower + m_upper) / 2.0; }
    double binSize() const { return m_upper - m_lower; }
};

//! Checks if value is contained in bin:
//! value in [m_lower, m_upper)
bool BinContains(const Bin1D& bin, double value);

//! An one-dimensional range of R3's.

struct Bin1DKVector {
    Bin1DKVector() = default;
    Bin1DKVector(const R3 lower, const R3 upper) : m_q_lower(lower), m_q_upper(upper) {}
    Bin1DKVector(double wavelength, const Bin1D& alpha_bin, const Bin1D& phi_bin);

    R3 center() const { return (m_q_lower + m_q_upper) / 2.0; }
    R3 span() const { return m_q_upper - m_q_lower; }
    R3 m_q_lower; //!< lower bound of the bin
    R3 m_q_upper; //!< upper bound of the bin
};

//! An one-dimensional range of C3's.

class Bin1DCVector {
public:
    Bin1DCVector() = default;
    Bin1DCVector(C3 lower, C3 upper) : m_q_lower(lower), m_q_upper(upper) {}
    Bin1DCVector(double wavelength, const Bin1D& alpha_bin, const Bin1D& phi_bin);
    C3 center() const { return (m_q_lower + m_q_upper) / 2.0; }
    C3 span() const { return m_q_upper - m_q_lower; }
    C3 m_q_lower; //!< lower bound of the bin
    C3 m_q_upper; //!< upper bound of the bin
};

#endif // BORNAGAIN_BASE_AXIS_BIN_H
