//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Core/Background/ConstantBackground.h
//! @brief     Defines class ConstantBackground.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_CORE_BACKGROUND_CONSTANTBACKGROUND_H
#define BORNAGAIN_CORE_BACKGROUND_CONSTANTBACKGROUND_H

#include "Core/Background/IBackground.h"

//! Class representing a constant background signal
//!
//! @ingroup simulation

class ConstantBackground : public IBackground {
public:
    inline static const std::string class_name = "ConstantBackground";
    std::string className() const final { return class_name; }

    ConstantBackground(std::vector<double> P);
    ConstantBackground(double background_value);

    ConstantBackground* clone() const override;

    double backgroundValue() const { return m_background_value; }

    void accept(INodeVisitor* visitor) const override { visitor->visit(this); }

    double addBackground(double intensity) const override;

private:
    const double& m_background_value;
};

#endif // BORNAGAIN_CORE_BACKGROUND_CONSTANTBACKGROUND_H
