//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Data/AxesItems.h
//! @brief     Defines various axis items
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_DATA_AXESITEMS_H
#define BORNAGAIN_GUI_MODEL_DATA_AXESITEMS_H

#include "GUI/Model/Session/SessionItem.h"
#include "GUI/Model/Types/DoubleDescriptor.h"
#include <memory>

class IAxis;

class BasicAxisItem : public SessionItem {
private:
    static constexpr auto P_IS_VISIBLE{"Visibility"};
    static constexpr auto P_NBINS{"Nbins"};
    static constexpr auto P_MIN_DEG{"Min [deg]"};
    static constexpr auto P_MAX_DEG{"Max [deg]"};
    static constexpr auto P_TITLE{"Title"};
    static constexpr auto P_TITLE_IS_VISIBLE{"Title Visibility"};

public:
    static constexpr auto M_TYPE{"BasicAxis"};

    BasicAxisItem();
    ~BasicAxisItem() override;

    int binCount() const;
    void setBinCount(int value);
    SessionItem* binsItem() const;

    DoubleDescriptor lowerBound() const;
    void setLowerBound(double value);
    SessionItem* lowerBoundItem() const;

    DoubleDescriptor upperBound() const;
    void setUpperBound(double value);
    SessionItem* upperBoundItem() const;

    /// either lower bound or upper bound property name
    static bool isBoundsPropertiesName(const QString& name);

    QString title() const;
    void setTitle(const QString& title);
    SessionItem* titleItem() const;
    static bool isTitlePropertyName(const QString& name);
    bool isTitleVisible() const;
    static bool isTitleVisiblePropertyName(const QString& name);

    virtual std::unique_ptr<IAxis> createAxis(double scale) const;

    SessionItem* visibilityItem() const;
    bool visibilityValue() const;
    void setVisibilityValue(bool b);
    static bool isVisibilityPropertyName(const QString& name);

protected:
    explicit BasicAxisItem(const QString& type);
    void register_basic_properties();
};

class AmplitudeAxisItem : public BasicAxisItem {
private:
    static constexpr auto P_IS_LOGSCALE{"log10"};
    static constexpr auto P_LOCK_MIN_MAX{"Lock (min, max)"};

public:
    static constexpr auto M_TYPE{"AmplitudeAxis"};

    AmplitudeAxisItem();

    bool isLogScale() const;
    void setLogScale(bool value);
    SessionItem* logScaleItem() const;
    static bool isLogScalePropertyName(const QString& name);

    bool isLocked() const;
    void setLocked(bool locked);

private:
    void setMinMaxEditor(const QString& editorType);
};

#endif // BORNAGAIN_GUI_MODEL_DATA_AXESITEMS_H
