//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Fit/ParameterTreeItems.h
//! @brief     Defines classes for ParameterTreeItems
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_FIT_PARAMETERTREEITEMS_H
#define BORNAGAIN_GUI_MODEL_FIT_PARAMETERTREEITEMS_H

#include "GUI/Model/Session/SessionItem.h"

//! ParameterTreeItems is a collection of items necessary to form a tuning tree for
//! real time widget.

//! The ParameterLabelItem class represents a label (string without value, like 'Layer',
//! 'MultiLayer') in a parameter tuning tree.

class ParameterLabelItem : public SessionItem {
public:
    static constexpr auto M_TYPE{"Parameter Label"};

    ParameterLabelItem();
};

//! The ParameterItem class represent a tuning value in a parameter tuning tree.

class ParameterItem : public SessionItem {
private:
    static constexpr auto P_LINK{"Link"};
    static constexpr auto P_BACKUP{"Backup"};

public:
    static constexpr auto M_TYPE{"Parameter"};

    ParameterItem();

    void propagateValueToLink(double newValue);
    SessionItem* linkedItem();
    void setLink(const QString& link);

    void restoreFromBackup();
    void setBackup(double value);
};

//! The ParameterContainerItem is a top item to hold all ParameterItem, represents an entry
//! point to parameter tuning tree. Part of JobItem.

class ParameterContainerItem : public SessionItem {
public:
    static constexpr auto M_TYPE{"Parameter Container"};

    ParameterContainerItem();
};

#endif // BORNAGAIN_GUI_MODEL_FIT_PARAMETERTREEITEMS_H
