//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/From/GUIDomainSampleVisitor.h
//! @brief     Defines class GUIDomainSampleVisitor
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_FROM_GUIDOMAINSAMPLEVISITOR_H
#define BORNAGAIN_GUI_MODEL_FROM_GUIDOMAINSAMPLEVISITOR_H

#include "Param/Node/INodeVisitor.h"
#include <QMap>
#include <QString>

class SampleModel;
class MaterialModel;
class SessionItem;
class Material;
class MaterialItem;

//! Visits domain sample tree to build GUI presentation.
//! Used from ObjectBuilder::populateSampleModel

class GUIDomainSampleVisitor : public INodeVisitor {
public:
    GUIDomainSampleVisitor();
    ~GUIDomainSampleVisitor() override;

    SessionItem* populateSampleModel(SampleModel* sampleModel, MaterialModel* materialModel,
                                     const MultiLayer& node, const QString& node_name = "");

    using INodeVisitor::visit;

    void visit(const ParticleLayout*) override;

    void visit(const Layer*) override;

    void visit(const MultiLayer*) override;

    void visit(const Particle*) override;
    void visit(const ParticleCoreShell*) override;
    void visit(const ParticleComposition*) override;
    void visit(const MesoCrystal*) override;
    void visit(const Crystal*) override;

    void visit(const FormFactorAnisoPyramid*) override;
    void visit(const FormFactorBarGauss*) override;
    void visit(const FormFactorBarLorentz*) override;
    void visit(const FormFactorBox*) override;
    void visit(const FormFactorCone*) override;
    void visit(const FormFactorCone6*) override;
    void visit(const FormFactorCuboctahedron*) override;
    void visit(const FormFactorCylinder*) override;
    void visit(const FormFactorDodecahedron*) override;
    void visit(const FormFactorEllipsoidalCylinder*) override;
    void visit(const FormFactorFullSphere*) override;
    void visit(const FormFactorFullSpheroid*) override;
    void visit(const FormFactorHemiEllipsoid*) override;
    void visit(const FormFactorIcosahedron*) override;
    void visit(const FormFactorPrism3*) override;
    void visit(const FormFactorPrism6*) override;
    void visit(const FormFactorPyramid*) override;
    void visit(const FormFactorCosineRippleBox*) override;
    void visit(const FormFactorCosineRippleGauss*) override;
    void visit(const FormFactorCosineRippleLorentz*) override;
    void visit(const FormFactorSawtoothRippleBox*) override;
    void visit(const FormFactorSawtoothRippleGauss*) override;
    void visit(const FormFactorSawtoothRippleLorentz*) override;
    void visit(const FormFactorTetrahedron*) override;
    void visit(const FormFactorTruncatedCube*) override;
    void visit(const FormFactorTruncatedSphere*) override;
    void visit(const FormFactorTruncatedSpheroid*) override;

    void visit(const Interference1DLattice*) override;
    void visit(const Interference2DLattice*) override;
    void visit(const Interference2DParaCrystal*) override;
    void visit(const InterferenceFinite2DLattice*) override;
    void visit(const InterferenceHardDisk*) override;
    void visit(const InterferenceRadialParaCrystal*) override;

    void visit(const RotationX*) override;
    void visit(const RotationY*) override;
    void visit(const RotationZ*) override;
    void visit(const RotationEuler*) override;

private:
    void buildAbundanceInfo(SessionItem* particleItem);
    MaterialItem* createMaterialFromDomain(const Material*);
    template <typename T> T* CreateIParticle(SessionItem* parent, const IParticle* particle);
    template <typename T> T* InsertIParticle(const IParticle* p_particle);

    QMap<int, SessionItem*> m_levelToParentItem;
    QMap<QString, double> m_propertyToValue;
    QMap<QString, bool> m_sample_encountered;
    QMap<SessionItem*, const ISampleNode*> m_itemToSample;
    QString m_topSampleName;

    SampleModel* m_sampleModel;
    MaterialModel* m_materialModel;
};

#endif // BORNAGAIN_GUI_MODEL_FROM_GUIDOMAINSAMPLEVISITOR_H
