//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Instrument/InstrumentModel.h
//! @brief     Defines class InstrumentModel
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_INSTRUMENT_INSTRUMENTMODEL_H
#define BORNAGAIN_GUI_MODEL_INSTRUMENT_INSTRUMENTMODEL_H

#include "GUI/Model/Session/SessionModel.h"
#include <functional>

class InstrumentItem;
class Instrument2DItem;

class InstrumentModel : public SessionModel {
    Q_OBJECT

public:
    explicit InstrumentModel(QObject* parent = nullptr);
    ~InstrumentModel() override;

    InstrumentModel* createCopy(SessionItem* parent = nullptr) override;
    QVector<SessionItem*> nonXMLItems() const override;
    void readFrom(QXmlStreamReader* reader, MessageService* messageService = nullptr) override;

    //! Inserts a deep copy (also of any non xml data in a pointwise axis)
    //! The id will not be copied, but a new unique one will be created
    //! Returns the newly created instrument.
    InstrumentItem* insertCopy(const InstrumentItem& source);

    QVector<InstrumentItem*> instrumentItems() const;
    QVector<Instrument2DItem*> instrument2DItems() const;

    InstrumentItem* findInstrumentById(const QString& instrumentId) const;
    bool instrumentExists(const QString& instrumentId) const;

    void removeInstrument(InstrumentItem* instrument);

    QString suggestInstrumentName(const QString& baseName) const;

    QVector<InstrumentItem*>
    instrumentItems(std::function<bool(const InstrumentItem*)> accept) const;

signals:
    void instrumentAddedOrRemoved();
    void instrumentNameChanged(const InstrumentItem* instrument);

    //! Signals any change in the settings of the given instrument
    void instrumentChanged(const InstrumentItem* instrument);

private:
    void onRowsChange(const QModelIndex& parent, int, int);
    void onInstrumentPropertyChange(const InstrumentItem* instrument, const QString& propertyName);
};

#endif // BORNAGAIN_GUI_MODEL_INSTRUMENT_INSTRUMENTMODEL_H
