//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Material/MaterialModel.h
//! @brief     Defines class MaterialModel
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_MATERIAL_MATERIALMODEL_H
#define BORNAGAIN_GUI_MODEL_MATERIAL_MATERIALMODEL_H

#include "GUI/Model/Session/SessionModel.h"

class MaterialItem;

class MaterialModel : public SessionModel {
    Q_OBJECT

public:
    explicit MaterialModel(QObject* parent = nullptr);
    ~MaterialModel() override;

    void clear() override;

    MaterialModel* createCopy(SessionItem* parent = nullptr) override;

    MaterialItem* addRefractiveMaterial(const QString& name, double delta, double beta);
    MaterialItem* addSLDMaterial(const QString& name, double sld, double abs_term);

    MaterialItem* materialFromName(const QString& name) const;
    MaterialItem* materialFromIdentifier(const QString& identifier) const;

    //! Returns clone of given material. Clone will get the same material identifier!
    MaterialItem* cloneMaterial(MaterialItem* material);

    QVector<MaterialItem*> materialItems() const;
    MaterialItem* findMaterialItem(const QString& identifier) const;

    MaterialItem* defaultMaterial() const;

    void removeMaterial(const QString& identifier);
    void removeMaterial(MaterialItem* materialItem);

    void readFrom(QXmlStreamReader* reader, MessageService* messageService = nullptr) override;
    void initFrom(SessionModel* model, SessionItem* parent) override;

    //! Compares for complete equality (same material identifiers, same order of materials,...)
    bool operator==(const MaterialModel& other) const;
    bool operator!=(const MaterialModel& other) const;

signals:
    void materialChanged(MaterialItem* materialItem);

private:
    void onRowsChange(const QModelIndex& parent, int, int);
};

#endif // BORNAGAIN_GUI_MODEL_MATERIAL_MATERIALMODEL_H
