//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Material/MaterialModelStore.cpp
//! @brief     Implements namespace MaterialModelStore
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Material/MaterialModelStore.h"
#include "GUI/Util/Error.h"

namespace {
MaterialModel* theMaterialModel = nullptr;
}

namespace GUI::Model::MaterialModelStore {

MaterialModel* materialModel()
{
    return theMaterialModel;
}

void subscribe(MaterialModel* materialModel)
{
    if (theMaterialModel != nullptr)
        throw Error("GUI::Model::MaterialModelStore::subscribe() -> Error. Attempt to subscribe "
                    "MaterialModel twice.");

    theMaterialModel = materialModel;
}

void unsubscribe(MaterialModel* materialModel)
{
    if (theMaterialModel != materialModel)
        throw Error("GUI::Model::MaterialModelStore::unsubscribe -> Error. Attempt to unsubscribe "
                    "MaterialModel before it was subscribed.");

    theMaterialModel = nullptr;
}
} // namespace GUI::Model::MaterialModelStore
