//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Sample/ItemWithMaterial.cpp
//! @brief     Implements class ItemWithMaterial
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Sample/ItemWithMaterial.h"
#include "GUI/Model/Job/JobItem.h"
#include "GUI/Model/Job/JobModelFunctions.h"
#include "GUI/Model/Material/MaterialItem.h"
#include "GUI/Model/Material/MaterialItemContainer.h"
#include "GUI/Model/Material/MaterialItemUtils.h"
#include "GUI/Model/Material/MaterialModel.h"
#include "GUI/Model/Material/MaterialModelStore.h"

void ItemWithMaterial::setMaterial(const MaterialItem* materialItem)
{
    setItemValue(P_MATERIAL, materialItem->identifier());
}

void ItemWithMaterial::setMaterial(const QString& materialIdentifier)
{
    setItemValue(P_MATERIAL, materialIdentifier);
}

bool ItemWithMaterial::isMaterialPropertyName(const QString& name)
{
    return name == P_MATERIAL;
}

void ItemWithMaterial::setMaterialUndefined()
{
    setItemValue(P_MATERIAL, QString());
}

QColor ItemWithMaterial::materialColor() const
{
    ASSERT(materialItem());
    return materialItem()->color();
}

QString ItemWithMaterial::materialName() const
{
    ASSERT(materialItem());
    return materialItem()->materialName();
}

QString ItemWithMaterial::materialIdentifier() const
{
    return getItemValue(P_MATERIAL).toString();
}

MaterialItem* ItemWithMaterial::materialItem() const
{
    if (materialIdentifier().isEmpty())
        return nullptr;

    const auto* const parentJob = GUI::Model::JobFunctions::findJobItem(this);
    if (parentJob)
        return parentJob->materialContainerItem()->findMaterialById(materialIdentifier());

    if (GUI::Model::MaterialModelStore::materialModel())
        return GUI::Model::MaterialModelStore::materialModel()->findMaterialItem(
            materialIdentifier());

    return nullptr;
}

ItemWithMaterial::ItemWithMaterial(const QString& model_type) : SessionItem(model_type)
{
    addProperty(P_MATERIAL, QString())
        ->setToolTip("Material this item is made of")
        .setEditorType(SessionItem::EDITOR_TYPE_MATERIAL);
}
