//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Sample/ItemWithMaterial.h
//! @brief     Defines abstract item with a material property
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_SAMPLE_ITEMWITHMATERIAL_H
#define BORNAGAIN_GUI_MODEL_SAMPLE_ITEMWITHMATERIAL_H

#include "GUI/Model/Session/SessionItem.h"
#include <memory>

class Material;
class MaterialItem;

class ItemWithMaterial : public virtual SessionItem {
private:
    static constexpr auto P_MATERIAL{"Material"};

public:
    static bool isMaterialPropertyName(const QString& name);

    //! Set the material this item shall use.
    //! Stores the identifier, not the pointer!
    void setMaterial(const MaterialItem* materialItem);

    //! Set the material this item shall use.
    //! Stores the given identifier, not a pointer to the material!
    void setMaterial(const QString& materialIdentifier);

    //! Set "no material defined"
    void setMaterialUndefined();

    QColor materialColor() const;
    QString materialName() const;
    QString materialIdentifier() const;

    //! Returns the material item this item links to.
    MaterialItem* materialItem() const;

protected:
    ItemWithMaterial(const QString& model_type);
};

#endif // BORNAGAIN_GUI_MODEL_SAMPLE_ITEMWITHMATERIAL_H
