//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Sample/SampleModel.cpp
//! @brief     Implements class SampleModel
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Sample/SampleModel.h"
#include "GUI/Model/Group/PropertyItem.h"
#include "GUI/Model/Material/MaterialItemUtils.h"
#include "GUI/Model/Sample/ItemWithMaterial.h"
#include "GUI/Model/Sample/MultiLayerItem.h"
#include "GUI/Model/Session/ModelUtils.h"
#include <QApplication>
#include <QFontMetrics>
#include <QPixmap>

SampleModel::SampleModel(QObject* parent) : SessionModel(GUI::Session::XML::SampleModelTag, parent)
{
    setObjectName(GUI::Session::XML::SampleModelTag);
    connect(this, &SampleModel::dataChanged, this, &SampleModel::onDataChanged);
}

SampleModel* SampleModel::createCopy(SessionItem* parent)
{
    auto* result = new SampleModel();
    result->initFrom(this, parent);
    return result;
}

QVariant SampleModel::data(const QModelIndex& index, int role) const
{
    if (!index.isValid())
        return QVariant();

    if (SessionItem* item = itemForIndex(index)) {

        if (role == Qt::DecorationRole && index.column() == SessionFlags::ITEM_VALUE) {
            const auto propertyName = item->itemName();
            const auto* itemWithMaterial = dynamic_cast<ItemWithMaterial*>(item->parent());

            if (ItemWithMaterial::isMaterialPropertyName(propertyName)
                && itemWithMaterial != nullptr) {
                const int size = qApp->fontMetrics().height();
                QPixmap pixmap(size, size);
                pixmap.fill(itemWithMaterial->materialColor());
                return pixmap;
            }
        }
    }

    return SessionModel::data(index, role);
}

MultiLayerItem* SampleModel::multiLayerItem()
{
    return topItem<MultiLayerItem>();
}

QVector<MultiLayerItem*> SampleModel::multiLayerItems() const
{
    return topItems<MultiLayerItem>();
}

QVector<ItemWithMaterial*> SampleModel::itemsWithMaterial() const
{
    QVector<ItemWithMaterial*> result;

    for (auto* m : topItems<MultiLayerItem>())
        result << m->itemsWithMaterial();

    return result;
}

void SampleModel::removeMultiLayer(MultiLayerItem* item)
{
    removeItem(item);
}

void SampleModel::onDataChanged(const QModelIndex& index, const QModelIndex&)
{
    // check whether it's an item, then emit the relevant signal
    auto* const item = itemForIndex(index);
    if (item)
        emit itemDataChanged(item);
}
