//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Trafo/TransformationItem.cpp
//! @brief     Implements class TransformationItem
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Trafo/TransformationItem.h"
#include "GUI/Model/Group/GroupInfo.h"
#include "GUI/Model/Trafo/RotationItems.h"

TransformationItem::TransformationItem() : SessionItem(M_TYPE)
{
    setToolTip("Rotation applied to particles");

    GroupInfo info;
    info.add(XRotationItem::M_TYPE, "X axis Rotation");
    info.add(YRotationItem::M_TYPE, "Y axis Rotation");
    info.add(ZRotationItem::M_TYPE, "Z axis Rotation");
    info.add(EulerRotationItem::M_TYPE, "Euler Rotation");
    info.setDefaultType(ZRotationItem::M_TYPE);

    addGroupProperty(P_ROT, info);
}

RotationItem* TransformationItem::rotationItem() const
{
    return dynamic_cast<RotationItem*>(getGroupItem(P_ROT));
}

void TransformationItem::setRotationType(const QString& modelType)
{
    setGroupProperty(P_ROT, modelType);
}
