//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Fit/JobMessagePanel.cpp
//! @brief     Implements class JobMessagePanel
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Fit/JobMessagePanel.h"
#include "GUI/View/Tool/mainwindow_constants.h"
#include <QScrollBar>
#include <QStackedWidget>
#include <QTextEdit>
#include <QVBoxLayout>

namespace {

//! text color for the given log level
QColor color(FitLogLevel level)
{
    switch (level) {
    case FitLogLevel::Default:
        return Qt::black;
    case FitLogLevel::Success:
        return Qt::darkBlue;
    case FitLogLevel::Highlight:
        return Qt::darkGreen;
    case FitLogLevel::Warning:
        return Qt::darkYellow;
    case FitLogLevel::Error:
        return Qt::darkRed;
    default:
        return Qt::red;
    }
}

} // namespace


JobMessagePanel::JobMessagePanel(QWidget* parent)
    : InfoPanel(parent), m_log(nullptr), m_plainLog(new QTextEdit)
{
    setWindowTitle(GUI::Constants::JobMessagePanelName);
    setObjectName("JobMessagePanel");

    m_plainLog->setReadOnly(true);
    //    m_plainLog->setMaximumBlockCount(100000);
    m_plainLog->setFont(QFont("Courier"));

    m_stackedWidget->addWidget(m_plainLog);

    setContentVisible(false);
}

void JobMessagePanel::onClearLog()
{
    m_plainLog->clear();
}

void JobMessagePanel::appendMessage(const FitLog::Message& message)
{
    QScrollBar* scrollbar = m_plainLog->verticalScrollBar();
    bool autoscroll = scrollbar->value() == scrollbar->maximum();
    //    m_plainLog->appendPlainText(message);
    m_plainLog->setTextColor(color(message.level));
    m_plainLog->append(QString::fromStdString(message.text));
    if (autoscroll) {
        QTextCursor c = m_plainLog->textCursor();
        c.movePosition(QTextCursor::End);
        m_plainLog->setTextCursor(c);
    }
}

void JobMessagePanel::setLog(FitLog* log)
{
    if (m_log)
        m_log->disconnect(this);
    m_log = log;
    m_plainLog->clear();
    if (m_log) {
        for (const auto& record : m_log->messages())
            appendMessage(record);
        connect(m_log, &FitLog::cleared, this, &JobMessagePanel::onClearLog);
        connect(m_log, &FitLog::messageAppended, this, &JobMessagePanel::appendMessage);
    }
}
