//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Instrument/FootprintCorrectionEditor.h
//! @brief     Defines class FootprintCorrectionEditor
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_INSTRUMENT_FOOTPRINTCORRECTIONEDITOR_H
#define BORNAGAIN_GUI_VIEW_INSTRUMENT_FOOTPRINTCORRECTIONEDITOR_H

#include <QWidget>

class SquareFootprintForm;
class GaussFootprintForm;
class SpecularBeamItem;
class QComboBox;
class QStackedLayout;

//! FootprintCorrection editor (i.e. background) for instrument editors.
//! Operates on InstrumentItem.

class FootprintCorrectionEditor : public QWidget {
    Q_OBJECT

public:
    FootprintCorrectionEditor(QWidget* parent = nullptr, Qt::WindowFlags f = Qt::WindowFlags());
    void setItem(SpecularBeamItem* item);
    void clearItem();

private slots:
    void onFootprintSelected(int index);

private:
    QStackedLayout* m_stack;
    QComboBox* m_combo;
    QWidget* m_blankForm;
    SquareFootprintForm* m_squareForm;
    GaussFootprintForm* m_gaussForm;
    SpecularBeamItem* m_item;
};

#endif // BORNAGAIN_GUI_VIEW_INSTRUMENT_FOOTPRINTCORRECTIONEDITOR_H
