//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Instrument/OffSpecularInstrumentEditor.cpp
//! @brief     Implements class OffSpecularInstrumentEditor
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Instrument/OffSpecularInstrumentEditor.h"
#include "GUI/Model/Instrument/InstrumentItems.h"
#include "GUI/View/Instrument/Detail/CreateDetails.h"
#include "GUI/View/Instrument/DetectorEditor.h"
#include "GUI/View/Instrument/OffSpecularBeamEditor.h"
#include "GUI/View/Instrument/PolarizationAnalysisEditor.h"

#include <QVBoxLayout>

OffSpecularInstrumentEditor::OffSpecularInstrumentEditor(QWidget* parent, Qt::WindowFlags f)
    : QWidget(parent, f)
{
    auto* layout = new QVBoxLayout(this);

    m_beamEditor = new OffSpecularBeamEditor(this);
    layout->addWidget(GUI::Detail::createDetailsWidget(m_beamEditor, "Beam parameters"));

    m_detectorEditor = new DetectorEditor(this);
    layout->addWidget(GUI::Detail::createDetailsWidget(m_detectorEditor, "Detector parameters"));

    m_polpairAnalysisEditor = new PolarizationAnalysisEditor(this);
    layout->addWidget(GUI::Detail::createDetailsWidget(
        m_polpairAnalysisEditor, "Polarization analysis", /*expanded*/ false));

    layout->addStretch();
}

void OffSpecularInstrumentEditor::setItem(OffSpecularInstrumentItem* item)
{
    m_beamEditor->setItem(item);
    m_detectorEditor->setItem(item);
    m_polpairAnalysisEditor->setItem(item);
}
