//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Instrument/ResolutionFunctionEditor.h
//! @brief     A widget for editing the alignment of a detector
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_INSTRUMENT_RESOLUTIONFUNCTIONEDITOR_H
#define BORNAGAIN_GUI_VIEW_INSTRUMENT_RESOLUTIONFUNCTIONEDITOR_H

#include <QWidget>

class QComboBox;
class QStackedLayout;

class DetectorItem;
class GaussResolutionFunctionForm;


/// Widget for selecting the resolution function of a detector (combo box) and input of the
/// corresponding values

class ResolutionFunctionEditor : public QWidget {

public:
    enum Unit { Degree, Millimeter };
    ResolutionFunctionEditor(Unit unit, QWidget* parent = nullptr,
                             Qt::WindowFlags f = Qt::WindowFlags());
    void setItem(DetectorItem* item);
    void clearItem();

private slots:
    void onResolutionFunctionSelected(int index);

private:
    QComboBox* m_combo;
    QStackedLayout* m_stack;
    QWidget* m_blankForm;
    GaussResolutionFunctionForm* m_gaussForm;
    DetectorItem* m_item;
};

#endif // BORNAGAIN_GUI_VIEW_INSTRUMENT_RESOLUTIONFUNCTIONEDITOR_H
