//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Instrument/SphericalDetectorEditor.cpp
//! @brief     Implement class SphericalDetectorEditor
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Instrument/SphericalDetectorEditor.h"
#include "GUI/Model/Instrument/SphericalDetectorItem.h"
#include "GUI/View/Instrument/ResolutionFunctionEditor.h"
#include "GUI/View/Instrument/SphericalAxisEditor.h"

#include <QGridLayout>


//==================================================================================================
// SphericalDetectorEditor
//==================================================================================================

SphericalDetectorEditor::SphericalDetectorEditor(QWidget* parent, Qt::WindowFlags f)
    : QWidget(parent, f)
{
    auto* grid = new QGridLayout(this);
    grid->setColumnStretch(0, 1);
    grid->setColumnStretch(1, 1);
    grid->setColumnStretch(2, 1);

    m_phiAxisEditor = new SphericalAxisEditor(u8"\u03c6 axis", this);
    grid->addWidget(m_phiAxisEditor, 1, 0);

    m_alphaAxisEditor = new SphericalAxisEditor(u8"\u03b1 axis", this);
    grid->addWidget(m_alphaAxisEditor, 1, 1);

    m_resolutionFunctionEditor =
        new ResolutionFunctionEditor(ResolutionFunctionEditor::Degree, this);
    grid->addWidget(m_resolutionFunctionEditor, 1, 2);

    grid->setRowStretch(2, 1);
}

void SphericalDetectorEditor::setItem(SphericalDetectorItem* item)
{
    if (item) {
        m_phiAxisEditor->setItem(item->phiAxisItem());
        m_alphaAxisEditor->setItem(item->alphaAxisItem());
        m_resolutionFunctionEditor->setItem(item);
    } else {
        m_phiAxisEditor->clearItem();
        m_alphaAxisEditor->clearItem();
        m_resolutionFunctionEditor->clearItem();
    }
}
