//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/PropertyEditor/IntensityDataPropertyWidget.h
//! @brief     Defines class IntensityDataPropertyWidget
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_PROPERTYEDITOR_INTENSITYDATAPROPERTYWIDGET_H
#define BORNAGAIN_GUI_VIEW_PROPERTYEDITOR_INTENSITYDATAPROPERTYWIDGET_H

#include "GUI/Model/Group/SelectionDescriptor.h"
#include <QList>
#include <QString>
#include <QWidget>
#include <functional>

class IntensityDataItem;
class SessionItem;
class QFormLayout;
class DoubleDescriptor;

using std::function;

//! Widget to edit properties of an IntensityDataItem.

class IntensityDataPropertyWidget : public QWidget {
    Q_OBJECT
public:
    explicit IntensityDataPropertyWidget(QWidget* parent = nullptr);

    void setItem(IntensityDataItem* item);

private:
    QWidget* createDoubleSpinbox(DoubleDescriptor d);
    QWidget* createTextEdit(SessionItem* item);
    QWidget* createCheckBox(const QString& title, SessionItem* item);
    QWidget* createCheckBox(const QString& title, function<bool()> getter,
                            function<void(bool)> setter);
    QWidget* createComboBox(SelectionDescriptor<QString> d);

    void updateUIValues();

private:
    IntensityDataItem* m_item;
    QFormLayout* m_mainLayout;
    QList<function<void()>> m_updaters;
};

#endif // BORNAGAIN_GUI_VIEW_PROPERTYEDITOR_INTENSITYDATAPROPERTYWIDGET_H
