//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/SampleDesigner/LayerEditorUtils.h
//! @brief     Defines class LayerEditorUtils
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_SAMPLEDESIGNER_LAYEREDITORUTILS_H
#define BORNAGAIN_GUI_VIEW_SAMPLEDESIGNER_LAYEREDITORUTILS_H

#include "GUI/Model/Types/DoubleDescriptor.h"

#include <QColor>
#include <QList>
#include <functional>

class DoubleSpinBox;
class FTDecayFunction1DItem;
class FTDistribution1DItem;
class FormFactorItem;
class ItemWithParticles;
class LayerBasicRoughnessItem;
class LayerZeroRoughnessItem;
class QGridLayout;
class QLabel;
class QPushButton;
class QString;
class QWidget;
class RotationItem;
class SampleEditorController;
class VectorDescriptor;
class FTDecayFunction2DItem;
class FTDistribution2DItem;

//! Utility functions to support layer oriented sample editor
namespace LayerEditorUtils {

void updateLabelUnit(QLabel* label);
void updateLabelUnit(QLabel* label, DoubleSpinBox* editor);

//! Create DoubleSpinBoxes for the DoubeDescriptors and connect them to
//! SampleEditorController::setDouble()
void addMultiPropertyToGrid(QGridLayout* m_gridLayout, int firstCol,
                            const DoubleDescriptors& valueDescriptors, SampleEditorController* ec,
                            bool vertically, bool addSpacer);

//! Create DoubleSpinBoxes for the DoubeDescriptors and connect them to the given setNewValue()
void addMultiPropertyToGrid(QGridLayout* m_gridLayout, int firstCol,
                            const DoubleDescriptors& valueDescriptors,
                            function<void(double, DoubleDescriptor)> setNewValue, bool vertically,
                            bool addSpacer);

void addMultiPropertyToGrid(QGridLayout* m_gridLayout, int firstCol,
                            const DoubleDescriptors& valueDescriptors, SampleEditorController* ec,
                            bool addSpacer);

//! Create DoubleSpinBoxes for the DoubeDescriptors and connect them to
//! SampleEditorController::setDouble()
void addVectorToGrid(QGridLayout* m_gridLayout, int firstCol, const VectorDescriptor& v,
                     SampleEditorController* ec, bool vertically, bool addSpacer);

//! Create DoubleSpinBoxes for the DoubeDescriptors and connect them to the given setNewValue()
void addVectorToGrid(QGridLayout* m_gridLayout, int firstCol, const VectorDescriptor& v,
                     function<void(double, DoubleDescriptor)> setNewValue, bool vertically,
                     bool addSpacer);


QLabel* createBoldLabel(const QString& text);

DoubleDescriptors doubleDescriptorsOfItem(
    const std::variant<LayerZeroRoughnessItem*, LayerBasicRoughnessItem*>& item);

DoubleDescriptors doubleDescriptorsOfItem(FTDecayFunction1DItem* item);
DoubleDescriptors doubleDescriptorsOfItem(FTDecayFunction2DItem* item);
DoubleDescriptors doubleDescriptorsOfItem(FormFactorItem* item);
DoubleDescriptors doubleDescriptorsOfItem(FTDistribution1DItem* item);
DoubleDescriptors doubleDescriptorsOfItem(FTDistribution2DItem* item);
DoubleDescriptors doubleDescriptorsOfItem(RotationItem* item);

QWidget* createWidgetForItemWithParticles(QWidget* parentWidget,
                                          ItemWithParticles* itemWithParticles, bool allowAbundance,
                                          SampleEditorController* ec, bool allowRemove = true);

QPushButton* createAddParticleButton(QWidget* parentWidget,
                                     std::function<void(const QString&)> slot);

QList<QColor> predefinedLayerColors();

} // namespace LayerEditorUtils

#endif // BORNAGAIN_GUI_VIEW_SAMPLEDESIGNER_LAYEREDITORUTILS_H
