//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/SampleDesigner/ScriptPanel.h
//! @brief     Defines class SampleDesigner
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_SAMPLEDESIGNER_SCRIPTPANEL_H
#define BORNAGAIN_GUI_VIEW_SAMPLEDESIGNER_SCRIPTPANEL_H

#include "GUI/View/Common/InfoPanel.h"

class SampleModel;
class PythonSyntaxHighlighter;
class UpdateTimer;
class CautionSign;
class QShowEvent;
class QHideEvent;
class QTextEdit;
class QModelIndex;

//! Resides at the bottom of SampleView and displays a Python script.

class ScriptPanel : public InfoPanel {
    Q_OBJECT

public:
    explicit ScriptPanel(SampleModel* sampleModel, QWidget* parent);

protected:
    void showEvent(QShowEvent*) override;
    void hideEvent(QHideEvent*) override;

private:
    //! generates string representing code snippet for all multi layers in the model
    QString generateCodeSnippet();

    //! Update the editor with the script content
    void updateEditor();

    void setEditorConnected(bool isConnected);

    QTextEdit* m_textEdit;
    SampleModel* m_sampleModel;
    PythonSyntaxHighlighter* m_highlighter;
    UpdateTimer* m_updateTimer;
    CautionSign* m_cautionSign;
};

#endif // BORNAGAIN_GUI_VIEW_SAMPLEDESIGNER_SCRIPTPANEL_H
