//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Toplevel/ProjectSettingsView.h
//! @brief     Defines class ProjectSettingsView
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_TOPLEVEL_PROJECTSETTINGSVIEW_H
#define BORNAGAIN_GUI_VIEW_TOPLEVEL_PROJECTSETTINGSVIEW_H

#include "GUI/Model/Project/ProjectDocument.h"
#include <QWidget>

namespace Ui {
class ProjectSettingsView;
}

//! Widget to define project settings
class ProjectSettingsView : public QWidget {
    Q_OBJECT

public:
    ProjectSettingsView(QWidget* parentWidget, ProjectDocument* document);

protected:
    void showEvent(QShowEvent*) override;

private:
    //! Write entries in OptionsItem to UI
    void writeOptionsToUI();
    void writeFunctionalityToUI();
    void updateInformation();
    void storeAsDefaults();

    void onSingleInstrumentRadioToggled(bool newState);
    void onSingleSampleRadioToggled(bool newState);
    void onFunctionalityToggled();

    ProjectDocument::Functionalities functionalities() const;

private:
    Ui::ProjectSettingsView* m_ui;
    ProjectDocument* m_document;
};

#endif // BORNAGAIN_GUI_VIEW_TOPLEVEL_PROJECTSETTINGSVIEW_H
