//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Resample/Element/SpecularElement.cpp
//! @brief     Implements the class SpecularElement.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Resample/Element/SpecularElement.h"
#include "Base/Vector/Direction.h"
#include "Resample/Slice/KzComputation.h"

SpecularElement::SpecularElement(double kz, const PolMatrices& polpair, bool computable)
    : IElement(polpair)
    , m_intensity(0.0)
    , m_computable(computable)
    , m_kz_computation(
          [kz](const SliceStack& slices) { return Compute::Kz::computeKzFromSLDs(slices, kz); })
{
}

SpecularElement::SpecularElement(double wavelength, double alpha, const PolMatrices& polpair,
                                 bool computable)
    : IElement(polpair)
    , m_intensity(0.0)
    , m_computable(computable)
    , m_kz_computation([k = vecOfLambdaAlphaPhi(wavelength, alpha, 0.0)](const SliceStack& slices) {
        return Compute::Kz::computeKzFromRefIndices(slices, k);
    })
{
}

SpecularElement::SpecularElement(SpecularElement&& other) noexcept = default;

SpecularElement::~SpecularElement() = default;

std::vector<complex_t> SpecularElement::produceKz(const SliceStack& slices)
{
    return m_kz_computation(slices);
}
