//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Resample/Element/SpecularElement.h
//! @brief     Declares the class SpecularElement.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifdef SWIG
#error no need to expose this header to Swig
#endif

#ifndef USER_API
#ifndef BORNAGAIN_RESAMPLE_ELEMENT_SPECULARELEMENT_H
#define BORNAGAIN_RESAMPLE_ELEMENT_SPECULARELEMENT_H

#include "Base/Element/IElement.h"
#include "Base/Types/Complex.h"
#include "Base/Vector/Vectors3D.h"
#include <memory>
#include <vector>

class SliceStack;

//! Data stucture containing both input and output of a single image pixel for specular simulation.
//! @ingroup simulation

class SpecularElement : public IElement {
public:
    SpecularElement(double kz, const PolMatrices& polpair, bool computable);
    SpecularElement(double wavelength, double alpha, const PolMatrices& polpair, bool computable);

    SpecularElement(const SpecularElement& other) = delete;
    SpecularElement(SpecularElement&& other) noexcept;

    ~SpecularElement();

    //! Returns assigned PolarizerPair.
    const PolMatrices& polarizationHandler() const { return m_polpair; }

    double intensity() const { return m_intensity; }
    void setIntensity(double intensity) { m_intensity = intensity; }

    //! Returns calculation flag (if it's false, zero intensity is assigned to the element)
    bool isCalculated() const { return m_computable; }

    //! Returns kz values for Abeles computation of reflection/transition coefficients
    std::vector<complex_t> produceKz(const SliceStack& slices);

private:
    double m_intensity; //!< simulated intensity for detector cell
    const bool m_computable;
    const std::function<std::vector<complex_t>(const SliceStack&)> m_kz_computation;
};

#endif // BORNAGAIN_RESAMPLE_ELEMENT_SPECULARELEMENT_H
#endif // USER_API
