//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Aggregate/Interference3DLattice.h
//! @brief     Defines class Interference3DLattice.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SAMPLE_AGGREGATE_INTERFERENCE3DLATTICE_H
#define BORNAGAIN_SAMPLE_AGGREGATE_INTERFERENCE3DLATTICE_H

#include "Sample/Aggregate/IInterference.h"
#include "Sample/Lattice/Lattice3D.h"

class IPeakShape;

//! Interference function of a 3D lattice.
//! @ingroup interference

class Interference3DLattice : public IInterference {
public:
    inline static const std::string class_name = "Interference3DLattice";
    std::string className() const final { return class_name; }

    Interference3DLattice(const Lattice3D& lattice);
    ~Interference3DLattice() override;

    Interference3DLattice* clone() const override;

    void accept(INodeVisitor* visitor) const override { visitor->visit(this); }

    void setPeakShape(const IPeakShape& peak_shape);

    const Lattice3D& lattice() const;

    bool supportsMultilayer() const override { return false; }

    std::vector<const INode*> getChildren() const override;

    void onChange() override;

private:
    double iff_without_dw(R3 q) const override;
    void initRecRadius();

    Lattice3D m_lattice; // TODO unique_ptr as in other Interference%s
    std::unique_ptr<IPeakShape> m_peak_shape;
    double m_rec_radius; //!< radius in reciprocal space defining the nearest q vectors to use
};

#endif // BORNAGAIN_SAMPLE_AGGREGATE_INTERFERENCE3DLATTICE_H
