//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Scattering/ZLimits.cpp
//! @brief     Defines class ZLimits.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Sample/Scattering/ZLimits.h"
#include <algorithm>
#include <cmath>
#include <stdexcept>

ZLimits::ZLimits(double _min, double _max) : m_zmin(_min), m_zmax(_max)
{
    if (_min == _max)
        throw std::runtime_error("Bug or invalid API call: Empty z interval");
    if (_min > _max)
        throw std::runtime_error("Bug or invalid API call: Negative z interval");
}

ZLimits::ZLimits() : ZLimits(-inf, inf) {}

bool ZLimits::isFinite() const
{
    return !std::isinf(m_zmin) && !std::isinf(m_zmax);
}

ZLimits ZLimits::enclosingInterval(const ZLimits& left, const ZLimits& right)
{
    return {std::min(left.zBottom(), right.zBottom()), std::max(left.zTop(), right.zTop())};
}

bool operator==(const ZLimits& left, const ZLimits& right)
{
    return (left.zBottom() == right.zBottom() && left.zTop() == right.zTop());
}

bool operator!=(const ZLimits& left, const ZLimits& right)
{
    return !(left == right);
}

std::ostream& operator<<(std::ostream& ostr, const ZLimits& limits)
{
    return ostr << "Lower: " << limits.zBottom() << ", Upper: " << limits.zTop();
}
