#include "GUI/Model/Sample/LayerItem.h"
#include "GUI/Model/Sample/MultiLayerItem.h"
#include "GUI/Model/Session/ModelUtils.h"
#include "GUI/Model/Session/SessionModel.h"
#include "GUI/Model/Types/VectorItem.h"
#include "Tests/GTestWrapper/google_test.h"
#include <QVector>

class TestModelUtils : public ::testing::Test {
public:
};

//! Testing top item names.

TEST_F(TestModelUtils, topItemNames)
{
    SessionModel model("TestModel");

    // testing empty list
    EXPECT_TRUE(GUI::Model::ItemUtils::topItemNames(&model).isEmpty());

    // inserting three top items
    auto* item = model.insertItem<MultiLayerItem>();
    item->setItemName("name1");
    model.insertItem<LayerItem>();
    item = model.insertItem<MultiLayerItem>();
    item->setItemName("name2");

    // checking names of items of certain type
    ASSERT_EQ(GUI::Model::ItemUtils::topItemNames(&model, MultiLayerItem::M_TYPE).size(), 2);
    EXPECT_EQ(GUI::Model::ItemUtils::topItemNames(&model, MultiLayerItem::M_TYPE).at(0), "name1");
    EXPECT_EQ(GUI::Model::ItemUtils::topItemNames(&model, MultiLayerItem::M_TYPE).at(1), "name2");

    // checking names of all top items
    QStringList expected = {"name1", "Layer", "name2"};
    EXPECT_EQ(GUI::Model::ItemUtils::topItemNames(&model), expected);
}


TEST_F(TestModelUtils, suggestName)
{
    using namespace GUI::Model::ItemUtils;

    EXPECT_EQ(suggestName({"A", "B", "C"}, "A"), "A (2)");
    EXPECT_EQ(suggestName({"A", "B", "C"}, "D"), "D");
    EXPECT_EQ(suggestName({"A (1)", "A (19)"}, "A"), "A (20)");
    EXPECT_EQ(suggestName({"A (x)"}, "A"), "A");

    EXPECT_EQ(suggestName({"A", "A (2)"}, "A (2)"), "A (3)");
    EXPECT_EQ(suggestName({"A", "A (X)"}, "A (X)"), "A (X) (2)");
    EXPECT_EQ(suggestName({"A", "A (3)"}, "A (2)"), "A (4)");
    EXPECT_EQ(suggestName({"A"}, "A (2)"), "A (2)");
    EXPECT_EQ(suggestName({"A"}, "A (7)"), "A (7)");

    EXPECT_EQ(suggestName({"A (0)"}, "A"), "A (1)");
    EXPECT_EQ(suggestName({"A, A (0)"}, "A"), "A (1)");
}
