#include "Sample/Particle/Particle.h"
#include "Base/Const/Units.h"
#include "Base/Math/Constants.h"
#include "Sample/HardParticle/FormFactorFullSphere.h"
#include "Sample/Material/MaterialFactoryFuncs.h"
#include "Sample/Scattering/FormFactorDecoratorMaterial.h"
#include "Sample/Scattering/Rotations.h"
#include "Tests/GTestWrapper/google_test.h"

class ParticleTest : public ::testing::Test {
};

TEST_F(ParticleTest, Constructors)
{
    Material mat = HomogeneousMaterial("Vacuum", 0, 0);
    FormFactorFullSphere sphere(1.0);
    RotationZ transform(45. * Units::deg);

    // construction with form factor
    std::unique_ptr<Particle> p2(new Particle(mat, sphere));
    EXPECT_EQ(mat, *p2->material());
    auto p2_formfactor = std::unique_ptr<IFormFactor>(p2->createFormFactor());
    EXPECT_TRUE(dynamic_cast<FormFactorDecoratorMaterial*>(p2_formfactor.get()));
    EXPECT_EQ(nullptr, p2->rotation());

    // construction with transformation
    std::unique_ptr<Particle> p3(new Particle(mat, sphere, transform));
    EXPECT_EQ(mat, *p3->material());
    auto p3_formfactor = std::unique_ptr<IFormFactor>(p3->createFormFactor());
    EXPECT_TRUE(dynamic_cast<FormFactorDecoratorMaterial*>(p3_formfactor.get()));
}
